# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: message.proto
# Protobuf Python Version: 4.25.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\rmessage.proto\"J\n\x05Route\x12\x10\n\x08route_id\x18\x01 \x01(\x0c\x12\x11\n\ttimestamp\x18\x02 \x01(\x03\x12\r\n\x05saddr\x18\x03 \x01(\t\x12\r\n\x05\x64\x61\x64\x64r\x18\x04 \x01(\t\"\x91\x02\n\x07Message\x12\n\n\x02id\x18\x01 \x01(\x0c\x12\x1b\n\x04kind\x18\x02 \x03(\x0e\x32\r.Message.Kind\x12!\n\x07routing\x18\x03 \x01(\x0b\x32\x10.Message.Routing\x12\r\n\x05topic\x18\x04 \x01(\t\x12\x0f\n\x07payload\x18\x05 \x01(\x0c\x1a\x41\n\x07Routing\x12\x0e\n\x06src_id\x18\x01 \x01(\x0c\x12\x0e\n\x06\x64st_id\x18\x02 \x01(\x0c\x12\x16\n\x06routes\x18\x03 \x03(\x0b\x32\x06.Route\"W\n\x04Kind\x12\r\n\tHANDSHAKE\x10\x00\x12\n\n\x06GOSSIP\x10\x01\x12\x07\n\x03SYN\x10\x02\x12\x07\n\x03\x41\x43K\x10\x03\x12\x07\n\x03PUB\x10\x04\x12\x07\n\x03SUB\x10\x05\x12\x07\n\x03REQ\x10\x06\x12\x07\n\x03RES\x10\x07\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'message_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  _globals['_ROUTE']._serialized_start=17
  _globals['_ROUTE']._serialized_end=91
  _globals['_MESSAGE']._serialized_start=94
  _globals['_MESSAGE']._serialized_end=367
  _globals['_MESSAGE_ROUTING']._serialized_start=213
  _globals['_MESSAGE_ROUTING']._serialized_end=278
  _globals['_MESSAGE_KIND']._serialized_start=280
  _globals['_MESSAGE_KIND']._serialized_end=367
# @@protoc_insertion_point(module_scope)
