class Gossip:
    def __init__(self, transport, fanout=None, peer_id=None): ...
    async def send(self, message, peer_id): ...
    async def send_ack(self, message): ...
    async def send_forward(self, message): ...
    async def send_gossip(self, message): ...
    async def recv(self): ...
    async def close(self): ...
